S3method(a1,ReferencePanel)
S3method(a1,ReferenceShard)
S3method(a1_hash64,ReferencePanel)
S3method(a1_hash64,ReferenceShard)
S3method(a2,ReferencePanel)
S3method(a2,ReferenceShard)
S3method(a2_hash64,ReferencePanel)
S3method(a2_hash64,ReferenceShard)
S3method(a1freq,LDPanel)
S3method(a1freq,LDShard)
S3method(annomat,AnnotationPanel)
S3method(annomat,AnnotationShard)
S3method(annonames,AnnotationPanel)
S3method(beta_vec,Sumstats)
S3method(beta_vec,SumstatsShard)
S3method(bp,ReferencePanel)
S3method(bp,ReferenceShard)
S3method(chr,ReferencePanel)
S3method(chr,ReferenceShard)
S3method(eaf_vec,Sumstats)
S3method(eaf_vec,SumstatsShard)
S3method(father_id,GenotypePanel)
S3method(fetch_genotypes,GenotypePanel)
S3method(fetch_genotypes_int8,GenotypePanel)
S3method(fid,GenotypePanel)
S3method(default_chrX_sex,LDPanel)
S3method(info_vec,Sumstats)
S3method(info_vec,SumstatsShard)
S3method(iid,GenotypePanel)
S3method(is_female,GenotypePanel)
S3method(is_male,GenotypePanel)
S3method(is_object_compatible,ReferencePanel)
S3method(is_object_compatible,default)
S3method(is_present,GenotypePanel)
S3method(is_present,GenotypeShard)
S3method(is_present,Sumstats)
S3method(is_present,SumstatsShard)
S3method(is_single_nucleotide_variant,ReferencePanel)
S3method(is_single_nucleotide_variant,ReferenceShard)
S3method(is_strand_ambiguous,ReferencePanel)
S3method(is_strand_ambiguous,ReferenceShard)
S3method(is_subject_present,GenotypePanel)
S3method(logpvec,Sumstats)
S3method(logpvec,SumstatsShard)
S3method(mother_id,GenotypePanel)
S3method(multiply_r2,LDPanel)
S3method(nvec,Sumstats)
S3method(nvec,SumstatsShard)
S3method(num_annot,AnnotationPanel)
S3method(num_annot,AnnotationShard)
S3method(num_sample,GenotypePanel)
S3method(num_snp,LDPanel)
S3method(num_snp,LDShard)
S3method(num_snp,ReferencePanel)
S3method(num_snp,ReferenceShard)
S3method(num_snp,AnnotationPanel)
S3method(num_snp,AnnotationShard)
S3method(num_snp,GenotypePanel)
S3method(num_snp,GenotypeShard)
S3method(num_snp,Sumstats)
S3method(num_snp,SumstatsShard)
S3method(ploidy_female,GenotypePanel)
S3method(ploidy_female,GenotypeShard)
S3method(ploidy_male,GenotypePanel)
S3method(ploidy_male,GenotypeShard)
S3method(print,GenotypePanel)
S3method(print,ReferencePanel)
S3method(print,ReferenceShard)
S3method(print,AnnotationPanel)
S3method(print,LDPanel)
S3method(print,Sumstats)
S3method(reference,LDPanel)
S3method(save_cache,AnnotationPanel)
S3method(save_cache,GenotypePanel)
S3method(save_cache,ReferencePanel)
S3method(save_cache,Sumstats)
S3method(se_vec,Sumstats)
S3method(se_vec,SumstatsShard)
S3method(select_annotations,AnnotationPanel)
S3method(select_shards,AnnotationPanel)
S3method(select_shards,GenotypePanel)
S3method(select_shards,LDPanel)
S3method(select_shards,ReferencePanel)
S3method(select_shards,Sumstats)
S3method(sex,GenotypePanel)
S3method(shard_offsets,ReferencePanel)
S3method(shard_offsets,AnnotationPanel)
S3method(shard_offsets,GenotypePanel)
S3method(shard_offsets,LDPanel)
S3method(shard_offsets,Sumstats)
S3method(shards,AnnotationPanel)
S3method(shards,GenotypePanel)
S3method(shards,LDPanel)
S3method(shards,ReferencePanel)
S3method(shards,Sumstats)
S3method(shards,default)
S3method(snp,ReferencePanel)
S3method(snp,ReferenceShard)
S3method(source_layout,GenotypePanel)
S3method(source_row0,GenotypePanel)
S3method(source_row0,GenotypeShard)
S3method(union_annotations,AnnotationPanel)
S3method(validate_checksums,ReferencePanel)
S3method(zvec,Sumstats)
S3method(zvec,SumstatsShard)
export(a1)
export(a1_hash64)
export(a2)
export(a2_hash64)
export(a1freq)
export(annomat)
export(annonames)
export(beta_vec)
export(bp)
export(chr)
export(create_annotation)
export(create_annotations)
export(create_sumstats)
export(default_chrX_sex)
export(eaf_vec)
export(father_id)
export(fast_prune)
export(fetch_genotypes)
export(fetch_genotypes_int8)
export(fid)
export(get_verbosity)
export(info_vec)
export(iid)
export(is_female)
export(is_male)
export(is_object_compatible)
export(is_present)
export(is_single_nucleotide_variant)
export(is_strand_ambiguous)
export(is_subject_present)
export(load_annotations)
export(load_annotations_cache)
export(load_genotype)
export(load_genotype_cache)
export(load_ld)
export(load_ld_reference)
export(load_reference)
export(load_reference_cache)
export(load_sumstats)
export(load_sumstats_cache)
export(logpvec)
export(mother_id)
export(multiply_r2)
export(nvec)
export(num_annot)
export(num_sample)
export(num_snp)
export(ploidy_female)
export(ploidy_male)
export(prepare_ld_npz_for_r)
export(reference)
export(save_cache)
export(save_annotations_cache)
export(save_genotype_cache)
export(save_reference_cache)
export(save_sumstats_cache)
export(se_vec)
export(select_annotations)
export(select_shards)
export(sex)
export(set_verbosity)
export(shard_offsets)
export(shards)
export(snp)
export(source_layout)
export(source_row0)
export(union_annotations)
export(validate_checksums)
export(validate_ld_distribution)
export(version)
export(zvec)
importFrom(Matrix,Matrix)
importFrom(Matrix,diag)
importFrom(Matrix,sparseMatrix)
importFrom(Matrix,t)
importFrom(bit64,integer64)
importFrom(digest,digest)
importFrom(jsonlite,fromJSON)
importFrom(methods,as)
