# statgen R Package

Statistical genetics data objects and loaders for R. This package is part of the larger `statgen` repository, which also includes Python and MATLAB/Octave implementations.

## Installation

### From GitHub (development)

```r
# Install devtools if needed
if (!require("devtools")) install.packages("devtools")

# Install from the R-package directory
devtools::install()

# Or from the repository root
devtools::install("R-package")
```

### From CRAN (release)

```r
install.packages("statgen")
```

## Documentation

- [Main statgen repository](https://github.com/precimed/statgen)
- Package vignette: `vignette("statgen")`
