\name{version}
\alias{version}
\title{Return the statgen Package Version}
\usage{
version()
}
\value{
A character scalar containing the installed package version.
}
\description{
Returns the installed statgen R package version.
}
\examples{
version()
}
